package main

import (
	"context"
	"net"

	"github.com/dghubble/oauth1"
	"github.com/dghubble/oauth1/twitter"
	"gitlab.mashroom.com/mashroom/backend/common/api/health"
	"gitlab.mashroom.com/mashroom/backend/common/generic/addr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/grpc"
	logger "gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/common/generic/metrics"
	"gitlab.mashroom.com/mashroom/backend/common/generic/tracer"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model/mongo"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/service"
	"gitlab.mashroom.com/mashroom/backend/oauth/version"
	"golang.org/x/oauth2"
	"golang.org/x/oauth2/facebook"
	"golang.org/x/oauth2/google"
	"golang.org/x/oauth2/instagram"
)

var (
	envMongoURL      = env.ResolveString("MONGO_URL", "mongodb:27017")
	envMongoUsername = env.ResolveString("MONGO_USERNAME", "extras-data")
	envMongoPassword = env.ResolveString("MONGO_PASSWORD", "extras-data")
	envMongoAuthDB   = env.ResolveString("MONGO_AUTH", "admin")
	envMongoDB       = env.ResolveString("MONGO_DATABASE", "extras-data")
	envMongoSSLMode  = env.ResolveBool("MONGO_SSL_MODE", false)
)

var (
	envFacebookClientID = env.ResolveString(
		"FACEBOOK_CLIENT_ID",
		"2100745146653802",
	)
	envFacebookClientSecret = env.ResolveString(
		"FACEBOOK_CLIENT_SECRET",
		"507847d11e5410343a92124d95316096",
	)
	envFacebookRedirectURL = env.ResolveString(
		"FACEBOOK_REDIRECT_URL",
		"https://78dd008f.ngrok.io/",
	)

	envInstagramClientID = env.ResolveString(
		"INSTAGRAM_CLIENT_ID",
		"d8ce1663732e4c58bf18fee9f4232635",
	)
	envInstagramClientSecret = env.ResolveString(
		"INSTAGRAM_CLIENT_SECRET",
		"a27879ff84534bccbbfa1c106ed81def",
	)
	envInstagramRedirectURL = env.ResolveString(
		"INSTAGRAM_REDIRECT_URL",
		"https://78dd008f.ngrok.io/",
	)

	envTwitterConsumerKey = env.ResolveString(
		"TWITTER_CONSUMER_KEY",
		"tBn2NZp4cmIrdIQEj0OUf2pJF",
	)
	envTwitterConsumerSecret = env.ResolveString(
		"TWITTER_CONSUMER_SECRET",
		"Hmjq5U5Wkqf3gRNlSpqKefZvg2liMuSsno6F6we06QntuqZ0O4",
	)
	envTwitterRedirectURL = env.ResolveString(
		"TWITTER_REDIRECT_URL",
		"https://78dd008f.ngrok.io/",
	)

	envGoogleClientID = env.ResolveString(
		"GOOGLE_CLIENT_ID",
		"604819778943-29csgikdkugcv0cabtja12rp4esd7khv.apps.googleusercontent.com",
	)
	envGoogleClientSecret = env.ResolveString(
		"GOOGLE_CLIENT_SECRET",
		"ggmzb9cHChhcw68X__Drz4y5",
	)
	envGoogleRedirectURL = env.ResolveString(
		"GOOGLE_REDIRECT_URL",
		"https://dev.env.mashroom.com",
	)
)

func main() {
	ctx := context.Background()

	defer tracer.Close()

	repository, err := mongo.NewRepository(ctx, &mongo.Config{
		URL:          envMongoURL,
		Username:     envMongoUsername,
		Password:     envMongoPassword,
		Database:     envMongoDB,
		AuthSource:   envMongoAuthDB,
		MongoSSLMode: envMongoSSLMode,
	})
	if err != nil {
		panic(err)
	}

	err = repository.Migrate(ctx)
	if err != nil {
		panic(err)
	}

	srv, err := service.NewService(ctx, &service.Config{
		Repository: repository,
		OAuth: service.OAuth{
			Facebook: service.OAuth2Config{
				OAuth2Provider: &oauth2.Config{
					ClientID:     envFacebookClientID,
					ClientSecret: envFacebookClientSecret,
					RedirectURL:  envFacebookRedirectURL,
					Endpoint:     facebook.Endpoint,
					Scopes:       []string{"email", "user_birthday", "user_age_range", "user_location"},
				},
				UserInfoURL: "https://graph.facebook.com/me?fields=id,email,first_name,last_name",
			},
			Instagram: service.OAuth2Config{
				OAuth2Provider: &oauth2.Config{
					ClientID:     envInstagramClientID,
					ClientSecret: envInstagramClientSecret,
					RedirectURL:  envInstagramRedirectURL,
					Endpoint:     instagram.Endpoint,
					Scopes:       []string{"basic"},
				},
				UserInfoURL: "https://api.instagram.com/v1/users/self/?access_token=%s",
			},
			Google: service.OAuth2Config{
				OAuth2Provider: &oauth2.Config{
					ClientID:     envGoogleClientID,
					ClientSecret: envGoogleClientSecret,
					RedirectURL:  envGoogleRedirectURL,
					Endpoint:     google.Endpoint,
					Scopes:       []string{"profile", "email"},
				},
				UserInfoURL: "https://oauth2.googleapis.com/tokeninfo?id_token=%s",
			},
			Twitter: service.OAuth1Config{
				OAuth1Provider: &oauth1.Config{
					ConsumerKey:    envTwitterConsumerKey,
					ConsumerSecret: envTwitterConsumerSecret,
					CallbackURL:    envTwitterRedirectURL,
					Endpoint:       twitter.AuthorizeEndpoint,
				},
			},
		},
	})
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("oauth")
	if err != nil {
		panic(err)
	}

	oauth.RegisterOAuthServiceServer(grpcServer, srv)

	logger.Infoln("oauth started")

	healthServer := version.NewHealthServer()
	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go func() {
		_ = metrics.ConfigureListenAndServe(grpcServer)
	}()

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
