// Package mongo provides mongo repository implementation
package mongo

import (
	"context"
	"crypto/tls"
	"fmt"

	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"

	"gitlab.mashroom.com/mashroom/backend/common/generic/migrations"
	migrationsMongo "gitlab.mashroom.com/mashroom/backend/common/generic/migrations/mongo"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
	v1 "gitlab.mashroom.com/mashroom/backend/oauth/internal/model/mongo/migrations/v1"
)

const (
	collectionTwitterToken        = "twitter"
	collectionTwitterTokenRequest = "twitterRequestPairs"
	collectionInstagramToken      = "instagram"
	collectionFacebookToken       = "facebook"
)

// Config mongo repository config
type Config struct {
	URL          string
	Username     string
	Password     string
	Database     string
	AuthSource   string
	MongoSSLMode bool
}

type repositoryImpl struct {
	client                        *mongo.Client
	database                      *mongo.Database
	collectionTwitterToken        *mongo.Collection
	collectionTwitterTokenRequest *mongo.Collection
	collectionInstagramToken      *mongo.Collection
	collectionFacebookToken       *mongo.Collection
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid repository config")
	}

	if config.URL == "" {
		return fmt.Errorf("empty url in repository config")
	}

	if config.Username == "" {
		return fmt.Errorf("empty username in repository config")
	}

	if config.Password == "" {
		return fmt.Errorf("empty password in repository config")
	}

	if config.Database == "" {
		return fmt.Errorf("empty database in repository config")
	}

	if config.AuthSource == "" {
		return fmt.Errorf("empty database in repository config")
	}

	return nil
}

// NewRepository returns new repository instance
func NewRepository(ctx context.Context, config *Config) (model.Repository, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	clientOptions := options.Client().ApplyURI(fmt.Sprintf("mongodb://%s", config.URL)).
		SetAuth(options.Credential{
			AuthSource:  config.AuthSource,
			Username:    config.Username,
			Password:    config.Password,
			PasswordSet: true,
		})

	if config.MongoSSLMode {
		clientOptions.SetTLSConfig(&tls.Config{})
	}

	client, err := mongo.NewClient(clientOptions)
	if err != nil {
		return nil, err
	}

	err = client.Connect(ctx)
	if err != nil {
		return nil, err
	}

	db := client.Database(config.Database)

	return &repositoryImpl{
		client:                        client,
		database:                      db,
		collectionTwitterToken:        db.Collection(collectionTwitterToken),
		collectionTwitterTokenRequest: db.Collection(collectionTwitterTokenRequest),
		collectionInstagramToken:      db.Collection(collectionInstagramToken),
		collectionFacebookToken:       db.Collection(collectionFacebookToken),
	}, nil
}

func (repo *repositoryImpl) Migrate(ctx context.Context) error {
	err := migrations.MigrateSchema(migrationsMongo.WithInstance(ctx, repo.database),
		&migrationsMongo.Migration{
			Version: "v1",
			Up:      v1.Up,
		},
	)
	if err != nil {
		return err
	}

	return nil
}

func (repo *repositoryImpl) TwitterTokenSave(
	ctx context.Context,
	m *model.TwitterToken,
) error {
	_, err := repo.collectionTwitterToken.UpdateOne(ctx, bson.M{"_id": m.ID}, m, options.Update().SetUpsert(true))

	return err
}

func (repo *repositoryImpl) TwitterTokenGetByID(
	ctx context.Context,
	id string,
) (m *model.TwitterToken, err error) {
	m = &model.TwitterToken{}
	err = repo.collectionTwitterToken.FindOne(ctx, bson.M{"_id": id}).Decode(m)

	return
}

func (repo *repositoryImpl) TwitterTokenRequestSave(
	ctx context.Context,
	m *model.TwitterRequestPair,
) error {
	_, err := repo.collectionTwitterTokenRequest.UpdateOne(ctx, bson.M{"_id": m.ID}, m, options.Update().SetUpsert(true))

	return err
}

func (repo *repositoryImpl) TwitterTokenRequestGetByID(
	ctx context.Context,
	id string,
) (m *model.TwitterRequestPair, err error) {
	m = &model.TwitterRequestPair{}
	err = repo.collectionTwitterTokenRequest.FindOne(ctx, bson.M{"_id": id}).Decode(m)

	return
}

func (repo *repositoryImpl) FacebookTokenSave(
	ctx context.Context,
	m *model.Oauth2Token,
) error {
	_, err := repo.collectionTwitterToken.UpdateOne(ctx, bson.M{"_id": m.ID}, m, options.Update().SetUpsert(true))

	return err
}

func (repo *repositoryImpl) FacebookTokenGetByID(
	ctx context.Context,
	id string,
) (m *model.Oauth2Token, err error) {
	m = &model.Oauth2Token{}
	err = repo.collectionTwitterTokenRequest.FindOne(ctx, bson.M{"_id": id}).Decode(m)

	return
}

func (repo *repositoryImpl) InstagramTokenSave(
	ctx context.Context,
	m *model.Oauth2Token,
) error {
	_, err := repo.collectionTwitterToken.UpdateOne(ctx, bson.M{"_id": m.ID}, m, options.Update().SetUpsert(true))

	return err
}

func (repo *repositoryImpl) InstagramTokenGetByID(
	ctx context.Context,
	id string,
) (m *model.Oauth2Token, err error) {
	m = &model.Oauth2Token{}
	err = repo.collectionTwitterTokenRequest.FindOne(ctx, bson.M{"_id": id}).Decode(m)

	return
}
