//go:generate mockgen -package=mock -destination=mock/oauth_provider.go -source=api.go OAuth2Provider,OAuth1Provider
//go:generate mockgen -package=mock -destination=mock/net_http_roundtripper.go net/http RoundTripper

// Package service provides implementation for oauth service
package service

import (
	"context"
	"fmt"
	"net/http"
	"net/url"

	"github.com/dghubble/oauth1"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
	"golang.org/x/oauth2"
)

// OAuth2Provider external provider
type OAuth2Provider interface {
	AuthCodeURL(ref string, opts ...oauth2.AuthCodeOption) string
	Client(ctx context.Context, token *oauth2.Token) *http.Client
	Exchange(ctx context.Context, code string, opts ...oauth2.AuthCodeOption) (*oauth2.Token, error)
}

// OAuth2Config for provider
type OAuth2Config struct {
	OAuth2Provider
	UserInfoURL string
}

// OAuth1Provider external provider
type OAuth1Provider interface {
	Client(ctx context.Context, token *oauth1.Token) *http.Client
	RequestToken() (token, secret string, err error)
	AuthorizationURL(token string) (*url.URL, error)
	AccessToken(reqtoken, reqsecret, verifier string) (token, secret string, err error)
}

// OAuth1Config for provider
type OAuth1Config struct {
	OAuth1Provider
}

// OAuth provides oauth services config details
type OAuth struct {
	Facebook  OAuth2Config
	Instagram OAuth2Config
	Google    OAuth2Config
	Twitter   OAuth1Config
}

// Config for service implementation
type Config struct {
	// Persistent repository
	Repository model.Repository
	OAuth      OAuth
}

// Service implementation
type Service interface {
	oauth.OAuthServiceServer
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid service config")
	}

	if config.Repository == nil {
		return fmt.Errorf("no repository in service config")
	}

	return nil
}

// NewService returns new service instance
func NewService(ctx context.Context, config *Config) (Service, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	srv := &serviceImpl{
		config: *config,
	}

	return srv, nil
}
