package service

import (
	"context"
	"encoding/json"
	"errors"
	"io/ioutil"

	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
	"go.mongodb.org/mongo-driver/mongo"
	"golang.org/x/oauth2"
)

func (srv *serviceImpl) facebookUserInfo(
	ctx context.Context,
	token *oauth2.Token,
) (*oauth.FacebookGetUserInfoResponse, error) {
	client := srv.config.OAuth.Facebook.Client(ctx, token)

	resp, err := client.Get(srv.config.OAuth.Facebook.UserInfoURL)
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = resp.Body.Close()
	}()

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}

	var data struct {
		ID        string `json:"id"`
		Email     string `json:"email"`
		FirstName string `json:"first_name"`
		LastName  string `json:"last_name"`
	}

	err = json.Unmarshal(body, &data)
	if err != nil {
		return nil, err
	}

	return &oauth.FacebookGetUserInfoResponse{
		Id:        data.ID,
		Email:     data.Email,
		FirstName: data.FirstName,
		LastName:  data.LastName,
		Raw:       body,
	}, nil
}

func (srv *serviceImpl) FacebookAuthUrl(
	ctx context.Context,
	req *oauth.FacebookAuthUrlRequest,
) (*oauth.FacebookAuthUrlResponse, error) {
	return &oauth.FacebookAuthUrlResponse{
		Url: srv.config.OAuth.Facebook.AuthCodeURL(req.GetRef(), oauth2.AccessTypeOffline),
	}, nil
}

func (srv *serviceImpl) FacebookExchangeToken(
	ctx context.Context,
	req *oauth.FacebookExchangeTokenRequest,
) (*oauth.FacebookExchangeTokenResponse, error) {
	token, err := srv.config.OAuth.Facebook.Exchange(ctx, req.GetCode())
	if err != nil {
		return nil, err
	}

	err = srv.config.Repository.FacebookTokenSave(ctx, &model.Oauth2Token{
		ID:    req.GetRef(),
		Token: token,
	})
	if err != nil {
		return nil, err
	}

	return &oauth.FacebookExchangeTokenResponse{}, nil
}

func (srv *serviceImpl) FacebookGetUserInfo(
	ctx context.Context,
	req *oauth.RefAccessRequest,
) (*oauth.FacebookGetUserInfoResponse, error) {
	token, err := srv.config.Repository.FacebookTokenGetByID(ctx, req.GetRef())
	if err != nil {
		if errors.Is(err, mongo.ErrNoDocuments) {
			return nil, errNotAuthenticated
		}

		return nil, err
	}

	return srv.facebookUserInfo(ctx, token.Token)
}

func (srv *serviceImpl) FacebookGetUserInfoWithToken(
	ctx context.Context,
	req *oauth.TokenAccessRequest,
) (*oauth.FacebookGetUserInfoResponse, error) {
	return srv.facebookUserInfo(ctx, &oauth2.Token{
		AccessToken: req.GetAccessToken(),
		TokenType:   "bearer",
	})
}
