package service

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"io/ioutil"

	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
	"go.mongodb.org/mongo-driver/mongo"
	"golang.org/x/oauth2"
)

func (srv *serviceImpl) instagramUserInfo(
	ctx context.Context,
	token *oauth2.Token,
) (*oauth.InstagramGetUserInfoResponse, error) {
	client := srv.config.OAuth.Instagram.Client(ctx, token)

	url := fmt.Sprintf(srv.config.OAuth.Instagram.UserInfoURL, token.AccessToken)

	resp, err := client.Get(url)
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = resp.Body.Close()
	}()

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}

	var data struct {
		Data struct {
			ID             string `json:"id"`
			Username       string `json:"username"`
			FullName       string `json:"full_name"`
			ProfilePicture string `json:"profile_picture"`
			Bio            string `json:"bio"`
			Website        string `json:"website"`
			IsBusiness     bool   `json:"is_business"`
			Counts         struct {
				Media      int `json:"media"`
				Follows    int `json:"follows"`
				FollowedBy int `json:"followed_by"`
			} `json:"counts"`
		} `json:"data"`
	}

	err = json.Unmarshal(body, &data)
	if err != nil {
		return nil, err
	}

	return &oauth.InstagramGetUserInfoResponse{
		Username:       data.Data.Username,
		FullName:       data.Data.FullName,
		ProfilePicture: data.Data.ProfilePicture,
		Raw:            body,
	}, nil
}

func (srv *serviceImpl) InstagramAuthUrl(
	ctx context.Context,
	req *oauth.InstagramAuthUrlRequest,
) (*oauth.InstagramAuthUrlResponse, error) {
	return &oauth.InstagramAuthUrlResponse{
		Url: srv.config.OAuth.Instagram.AuthCodeURL(req.GetRef(), oauth2.AccessTypeOffline),
	}, nil
}

func (srv *serviceImpl) InstagramExchangeToken(
	ctx context.Context,
	req *oauth.InstagramExchangeTokenRequest,
) (*oauth.InstagramExchangeTokenResponse, error) {
	token, err := srv.config.OAuth.Instagram.Exchange(ctx, req.GetCode())
	if err != nil {
		return nil, err
	}

	err = srv.config.Repository.InstagramTokenSave(ctx, &model.Oauth2Token{
		ID:    req.GetRef(),
		Token: token,
	})
	if err != nil {
		return nil, err
	}

	return &oauth.InstagramExchangeTokenResponse{}, nil
}

func (srv *serviceImpl) InstagramGetUserInfo(
	ctx context.Context,
	req *oauth.RefAccessRequest,
) (*oauth.InstagramGetUserInfoResponse, error) {
	token, err := srv.config.Repository.InstagramTokenGetByID(ctx, req.GetRef())
	if err != nil {
		if errors.Is(err, mongo.ErrNoDocuments) {
			return nil, errNotAuthenticated
		}

		return nil, err
	}

	return srv.instagramUserInfo(ctx, token.Token)
}

func (srv *serviceImpl) InstagramGetUserInfoWithToken(
	ctx context.Context,
	req *oauth.TokenAccessRequest,
) (*oauth.InstagramGetUserInfoResponse, error) {
	return srv.instagramUserInfo(ctx, &oauth2.Token{
		AccessToken: req.GetAccessToken(),
		TokenType:   "bearer",
	})
}
