package service

import (
	"context"
	"errors"

	"go.mongodb.org/mongo-driver/mongo"

	"github.com/dghubble/go-twitter/twitter"
	"github.com/dghubble/oauth1"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
)

func (srv *serviceImpl) TwitterAuthUrl(
	ctx context.Context,
	req *oauth.TwitterAuthUrlRequest,
) (*oauth.TwitterAuthUrlResponse, error) {
	requestToken, requestSecret, err := srv.config.OAuth.Twitter.RequestToken()
	if err != nil {
		return nil, err
	}

	err = srv.config.Repository.TwitterTokenRequestSave(ctx, &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  requestToken,
		RequestSecret: requestSecret,
	})
	if err != nil {
		return nil, err
	}

	url, err := srv.config.OAuth.Twitter.AuthorizationURL(requestToken)
	if err != nil {
		return nil, err
	}

	return &oauth.TwitterAuthUrlResponse{
		Url: url.String(),
	}, nil
}

func (srv *serviceImpl) TwitterExchangeToken(
	ctx context.Context,
	req *oauth.TwitterExchangeTokenRequest,
) (*oauth.TwitterExchangeTokenResponse, error) {
	pair, err := srv.config.Repository.TwitterTokenRequestGetByID(ctx, req.GetRef())
	if err != nil {
		return nil, err
	}

	accessToken, accessSecret, err := srv.config.OAuth.Twitter.AccessToken(
		pair.RequestToken,
		pair.RequestSecret,
		req.GetVerifier(),
	)
	if err != nil {
		return nil, err
	}

	token := oauth1.NewToken(accessToken, accessSecret)

	err = srv.config.Repository.TwitterTokenSave(ctx, &model.TwitterToken{
		ID:    req.GetRef(),
		Token: token,
	})
	if err != nil {
		return nil, err
	}

	return &oauth.TwitterExchangeTokenResponse{}, nil
}

func (srv *serviceImpl) TwitterGetUserInfo(
	ctx context.Context,
	req *oauth.RefAccessRequest,
) (*oauth.TwitterGetUserInfoResponse, error) {
	token, err := srv.config.Repository.TwitterTokenGetByID(ctx, req.GetRef())
	if err != nil {
		if errors.Is(err, mongo.ErrNoDocuments) {
			return nil, errNotAuthenticated
		}

		return nil, err
	}

	client := twitter.NewClient(srv.config.OAuth.Twitter.Client(ctx, token.Token))

	includeEmail := true

	user, resp, err := client.Accounts.VerifyCredentials(&twitter.AccountVerifyParams{
		IncludeEmail: &includeEmail,
	})
	if err != nil {
		return nil, err
	}

	_ = resp.Body.Close()

	return &oauth.TwitterGetUserInfoResponse{
		Name:  user.Name,
		Email: user.Email,
	}, nil
}

func (srv *serviceImpl) TwitterStatusUpdate(
	ctx context.Context,
	req *oauth.TwitterStatusUpdateRequest,
) (*oauth.TwitterStatusUpdateResponse, error) {
	token, err := srv.config.Repository.TwitterTokenGetByID(ctx, req.GetRef())
	if err != nil {
		return nil, err
	}

	client := twitter.NewClient(srv.config.OAuth.Twitter.Client(ctx, token.Token))

	_, resp, err := client.Statuses.Update(req.GetStatus(), nil)
	if err != nil {
		return nil, err
	}

	_ = resp.Body.Close()

	return &oauth.TwitterStatusUpdateResponse{}, nil
}
