package test

import (
	"errors"
	"io/ioutil"
	"net/http"
	"strings"
	"time"

	"github.com/golang/mock/gomock"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"golang.org/x/oauth2"
)

func (s *serviceSuite) TestGoogleSignIn_ok() {
	req := &oauth.GoogleSignInRequest{
		AuthorizationCode: "aaa1",
	}

	tok := &oauth2.Token{
		AccessToken:  "bbb1",
		TokenType:    "bbb2",
		RefreshToken: "bbb3",
		Expiry:       time.Unix(100, 10),
	}

	s.oauth2Provider.EXPECT().Exchange(s.testCtx, req.GetAuthorizationCode()).Return(tok, nil)

	s.oauth2Provider.EXPECT().Client(s.testCtx, tok).Return(s.client)

	info := `{
  "sub":         "ccc1",
  "email":       "ccc2",
  "given_name":  "ccc3",
  "family_name": "ccc4",
  "picture":     "ccc5"
}
`

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return &http.Response{
			Body: ioutil.NopCloser(strings.NewReader(info)),
		}, nil
	})

	resp, err := s.srv.GoogleSignIn(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.GoogleSignInResponse{
		Success: true,
		UserInfo: &oauth.GoogleUserInfo{
			Id:         "ccc1",
			Email:      "ccc2",
			GivenName:  "ccc3",
			FamilyName: "ccc4",
			PictureUrl: "ccc5",
		},
	}, resp)
}

func (s *serviceSuite) TestGoogleSignIn_error_client() {
	req := &oauth.GoogleSignInRequest{
		AuthorizationCode: "aaa1",
	}

	tok := &oauth2.Token{
		AccessToken:  "bbb1",
		TokenType:    "bbb2",
		RefreshToken: "bbb3",
		Expiry:       time.Unix(100, 10),
	}

	s.oauth2Provider.EXPECT().Exchange(s.testCtx, req.GetAuthorizationCode()).Return(tok, nil)

	s.oauth2Provider.EXPECT().Client(s.testCtx, tok).Return(s.client)

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return nil, errors.New("error")
	})

	_, err := s.srv.GoogleSignIn(s.testCtx, req)
	s.Require().Error(err)
}

func (s *serviceSuite) TestGoogleSignIn_error_exchange() {
	req := &oauth.GoogleSignInRequest{
		AuthorizationCode: "aaa1",
	}

	s.oauth2Provider.EXPECT().Exchange(s.testCtx, req.GetAuthorizationCode()).Return(nil, errors.New("error"))

	_, err := s.srv.GoogleSignIn(s.testCtx, req)
	s.Require().Error(err)
}
